@@ -32,7 +32,6 @@
 #include "gdk-pixbuf-scaled-anim.h"
 #include "gdk-pixbuf-loader.h"
 #include "gdk-pixbuf-marshal.h"
-#include "gdk-pixbuf-alias.h"
 
 enum {
         SIZE_PREPARED,
@@ -684,6 +683,9 @@ gdk_pixbuf_loader_get_animation (GdkPixbufLoader *loader)
  * to be finished, passing %NULL for @error to ignore it is
  * reasonable.
  *
+ * Remember that this does not unref the loader, so if you plan not to
+ * use it anymore, please g_object_unref() it.
+ *
  * Returns: %TRUE if all image data written so far was successfully
             passed out via the update_area signal
  **/
@@ -781,8 +783,3 @@ gdk_pixbuf_loader_get_format (GdkPixbufLoader *loader)
         else
                 return NULL;
 }
-
-
-#define __GDK_PIXBUF_LOADER_C__
-#include "gdk-pixbuf-aliasdef.c"
-
