@@ -1,3 +1,4 @@
+/* -*- mode: C; c-file-style: "linux" -*- */
 /* GdkPixbuf library - Windows Bitmap image loader
  *
  * Copyright (C) 1999 The Free Software Foundation
@@ -8,28 +9,30 @@
  * Based on io-ras.c
  *
  * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Library General Public
+ * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
  * version 2 of the License, or (at your option) any later version.
  *
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Library General Public License for more details.
+ * Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU Library General Public
+ * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the
  * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
  */
 
-#include <config.h>
+#include "config.h"
 #include <stdio.h>
-#include <stdlib.h>
+#ifdef HAVE_UNISTD_H
 #include <unistd.h>
+#endif
 #include <string.h>
 #include "gdk-pixbuf-private.h"
-#include "gdk-pixbuf-io.h"
+
+#define DUMPBIH 0
 
 
 
@@ -85,8 +88,7 @@ debugging purposes.
 
 */
 #if DUMPBIH
-static void
-DumpBIH(unsigned char *BIH)
+static void DumpBIH(unsigned char *BIH)
 {
 	printf("biSize      = %i \n",
 	       (int) (BIH[3] << 24) + (BIH[2] << 16) + (BIH[1] << 8) +
@@ -129,6 +131,7 @@ struct headerpair {
 	guint depth;
 	guint Negative;		/* Negative = 1 -> top down BMP,
 				   Negative = 0 -> bottom up BMP */
+	guint  n_colors;
 };
 
 /* Data needed for the "state" during decompression */
@@ -143,8 +146,9 @@ struct bmp_compression_state {
 /* Progressive loading */
 
 struct bmp_progressive_state {
-	ModulePreparedNotifyFunc prepared_func;
-	ModuleUpdatedNotifyFunc updated_func;
+	GdkPixbufModuleSizeFunc size_func;
+	GdkPixbufModulePreparedFunc prepared_func;
+	GdkPixbufModuleUpdatedFunc updated_func;
 	gpointer user_data;
 
 	ReadState read_state;
@@ -153,8 +157,9 @@ struct bmp_progressive_state {
 	guint Lines;		/* # of finished lines */
 
 	guchar *buff;
-	gint BufferSize;
-	gint BufferDone;
+	guint BufferSize;
+	guint BufferPadding;
+	guint BufferDone;
 
 	guchar (*Colormap)[3];
 
@@ -166,7 +171,7 @@ struct bmp_progressive_state {
 				   8  = 8 bpp colormapped
 				   1  = 1 bit bitonal
 				 */
-	gint Compressed;
+	guint Compressed;
 	struct bmp_compression_state compr;
 
 
@@ -176,55 +181,25 @@ struct bmp_progressive_state {
 	int r_mask, r_shift, r_bits;
 	int g_mask, g_shift, g_bits;
 	int b_mask, b_shift, b_bits;
+	int a_mask, a_shift, a_bits;
 
 	GdkPixbuf *pixbuf;	/* Our "target" */
 };
 
-gpointer gdk_pixbuf__bmp_image_begin_load (ModulePreparedNotifyFunc prepared_func,
-					   ModuleUpdatedNotifyFunc updated_func,
-					   ModuleFrameDoneNotifyFunc frame_done_func,
-					   ModuleAnimationDoneNotifyFunc
-					   anim_done_func, gpointer user_data);
+static gpointer
+gdk_pixbuf__bmp_image_begin_load(GdkPixbufModuleSizeFunc size_func,
+                                 GdkPixbufModulePreparedFunc prepared_func,
+				 GdkPixbufModuleUpdatedFunc updated_func,
+                                 gpointer user_data,
+                                 GError **error);
 
-void gdk_pixbuf__bmp_image_stop_load(gpointer data);
-gboolean gdk_pixbuf__bmp_image_load_increment(gpointer data, guchar *buf, guint size);
+static gboolean gdk_pixbuf__bmp_image_stop_load(gpointer data, GError **error);
+static gboolean gdk_pixbuf__bmp_image_load_increment(gpointer data,
+                                                     const guchar * buf,
+                                                     guint size,
+                                                     GError **error);
 
 
-
-/* Shared library entry point --> This should be removed when
-   generic_image_load enters gdk-pixbuf-io. */
-GdkPixbuf *
-gdk_pixbuf__bmp_image_load(FILE *f)
-{
-	guchar membuf[4096];
-	size_t length;
-	struct bmp_progressive_state *State;
-
-	GdkPixbuf *pb;
-
-	State = gdk_pixbuf__bmp_image_begin_load(NULL, NULL, NULL, NULL, NULL);
-
-	if (State == NULL)
-		return NULL;
-
-	while (feof(f) == 0) {
-		length = fread(membuf, 1, sizeof (membuf), f);
-		if (length > 0)
-			if (!gdk_pixbuf__bmp_image_load_increment (State, membuf, length)) {
-				gdk_pixbuf__bmp_image_stop_load (State);
-				return NULL;
-			}
-	}
-
-	if (State->pixbuf != NULL)
-		gdk_pixbuf_ref(State->pixbuf);
-
-	pb = State->pixbuf;
-
-	gdk_pixbuf__bmp_image_stop_load(State);
-	return pb;
-}
-
 /* Picks up a 32-bit little-endian integer starting at the specified location.
  * Does it by hand instead of dereferencing a simple (gint *) cast due to
  * alignment constraints many platforms.
@@ -242,55 +217,124 @@ lsb_16 (guchar *src)
 	return src[0] | (src[1] << 8);
 }
 
-static gboolean
-grow_buffer (struct bmp_progressive_state *State)
+static gboolean grow_buffer (struct bmp_progressive_state *State,
+                             GError **error)
 {
-  guchar *tmp = realloc (State->buff, State->BufferSize);
-  if (!tmp) {
+  guchar *tmp;
+
+  if (State->BufferSize == 0) {
+    g_set_error_literal (error,
+                         GDK_PIXBUF_ERROR,
+                         GDK_PIXBUF_ERROR_CORRUPT_IMAGE,
+                         _("BMP image has bogus header data"));
     State->read_state = READ_STATE_ERROR;
     return FALSE;
   }
+
+  tmp = g_try_realloc (State->buff, State->BufferSize);
+
+  if (!tmp) {
+    g_set_error_literal (error,
+                         GDK_PIXBUF_ERROR,
+                         GDK_PIXBUF_ERROR_INSUFFICIENT_MEMORY,
+                         _("Not enough memory to load bitmap image"));
+    State->read_state = READ_STATE_ERROR;
+    return FALSE;
+  }
+
   State->buff = tmp;
   return TRUE;
 }
 
 static gboolean
-DecodeHeader (unsigned char *BFH, unsigned char *BIH,
-	      struct bmp_progressive_state *State)
+decode_bitmasks (guchar *buf,
+		 struct bmp_progressive_state *State, 
+		 GError **error);
+
+static gboolean DecodeHeader(unsigned char *BFH, unsigned char *BIH,
+                             struct bmp_progressive_state *State,
+                             GError **error)
 {
-	g_assert (State->read_state == READ_STATE_HEADERS);
+	gint clrUsed;
+
+	/* First check for the two first bytes content. A sane
+	   BMP file must start with bytes 0x42 0x4D.  */
+	if (*BFH != 0x42 || *(BFH + 1) != 0x4D) {
+		g_set_error_literal (error,
+                                     GDK_PIXBUF_ERROR,
+                                     GDK_PIXBUF_ERROR_CORRUPT_IMAGE,
+                                     _("BMP image has bogus header data"));
+		State->read_state = READ_STATE_ERROR;
+		return FALSE;
+	}
 
         /* FIXME this is totally unrobust against bogus image data. */
-
 	if (State->BufferSize < lsb_32 (&BIH[0]) + 14) {
 		State->BufferSize = lsb_32 (&BIH[0]) + 14;
-		if (!grow_buffer (State))
+		if (!grow_buffer (State, error))
 			return FALSE;
 		return TRUE;
 	}
 
 #if DUMPBIH
 	DumpBIH(BIH);
-#endif
+#endif    
 
 	State->Header.size = lsb_32 (&BIH[0]);
-	if (State->Header.size == 40) {
+	if (State->Header.size == 124) {
+                /* BMP v5 */
+		State->Header.width = lsb_32 (&BIH[4]);
+		State->Header.height = lsb_32 (&BIH[8]);
+		State->Header.depth = lsb_16 (&BIH[14]);
+		State->Compressed = lsb_32 (&BIH[16]);
+	} else if (State->Header.size == 108) {
+                /* BMP v4 */
+		State->Header.width = lsb_32 (&BIH[4]);
+		State->Header.height = lsb_32 (&BIH[8]);
+		State->Header.depth = lsb_16 (&BIH[14]);
+		State->Compressed = lsb_32 (&BIH[16]);
+	} else if (State->Header.size == 64) {
+                /* BMP OS/2 v2 */
+		State->Header.width = lsb_32 (&BIH[4]);
+		State->Header.height = lsb_32 (&BIH[8]);
+		State->Header.depth = lsb_16 (&BIH[14]);
+		State->Compressed = lsb_32 (&BIH[16]);
+	} else if (State->Header.size == 40) {
+                /* BMP v3 */ 
 		State->Header.width = lsb_32 (&BIH[4]);
 		State->Header.height = lsb_32 (&BIH[8]);
 		State->Header.depth = lsb_16 (&BIH[14]);
 		State->Compressed = lsb_32 (&BIH[16]);
 	} else if (State->Header.size == 12) {
+                /* BMP OS/2 */
 		State->Header.width = lsb_16 (&BIH[4]);
 		State->Header.height = lsb_16 (&BIH[6]);
 		State->Header.depth = lsb_16 (&BIH[10]);
 		State->Compressed = BI_RGB;
 	} else {
-#if 0
-		g_set_error (error,
-			     GDK_PIXBUF_ERROR,
-			     GDK_PIXBUF_ERROR_CORRUPT_IMAGE,
-			     _("BMP image has unsupported header size"));
-#endif
+		g_set_error_literal (error,
+                                     GDK_PIXBUF_ERROR,
+                                     GDK_PIXBUF_ERROR_CORRUPT_IMAGE,
+                                     _("BMP image has unsupported header size"));
+		State->read_state = READ_STATE_ERROR;
+		return FALSE;
+	}
+
+	if (State->Header.size == 12)
+		clrUsed = 1 << State->Header.depth;
+	else
+		clrUsed = (int) (BIH[35] << 24) + (BIH[34] << 16) + (BIH[33] << 8) + (BIH[32]);
+
+	if (clrUsed != 0)
+		State->Header.n_colors = clrUsed;
+	else
+            State->Header.n_colors = (1 << State->Header.depth);
+	
+	if (State->Header.n_colors > (1 << State->Header.depth)) {
+		g_set_error_literal (error,
+                                     GDK_PIXBUF_ERROR,
+                                     GDK_PIXBUF_ERROR_CORRUPT_IMAGE,
+                                     _("BMP image has bogus header data"));
 		State->read_state = READ_STATE_ERROR;
 		return FALSE;
 	}
@@ -302,22 +346,27 @@ DecodeHeader (unsigned char *BFH, unsigned char *BIH,
 		State->Header.height = -State->Header.height;
 		State->Header.Negative = 1;
 	}
-	if (State->Header.width < 0) {
-		State->Header.width = -State->Header.width;
-		State->Header.Negative = 0;
+
+	if (State->Header.Negative && 
+	    (State->Compressed != BI_RGB && State->Compressed != BI_BITFIELDS))
+	{
+		g_set_error_literal (error,
+                                     GDK_PIXBUF_ERROR,
+                                     GDK_PIXBUF_ERROR_CORRUPT_IMAGE,
+                                     _("Topdown BMP images cannot be compressed"));
+		State->read_state = READ_STATE_ERROR;
+		return FALSE;
 	}
 
-	if (State->Header.width == 0 || State->Header.height == 0 ||
+	if (State->Header.width <= 0 || State->Header.height == 0 ||
 	    (State->Compressed == BI_RLE4 && State->Type != 4)    ||
 	    (State->Compressed == BI_RLE8 && State->Type != 8)	  ||
 	    (State->Compressed == BI_BITFIELDS && !(State->Type == 16 || State->Type == 32)) ||
-	    State->Compressed > BI_BITFIELDS) {
-#if 0
-		g_set_error (error,
-			     GDK_PIXBUF_ERROR,
-			     GDK_PIXBUF_ERROR_CORRUPT_IMAGE,
-			     _("BMP image has bogus header data"));
-#endif
+	    (State->Compressed > BI_BITFIELDS)) {
+		g_set_error_literal (error,
+                                     GDK_PIXBUF_ERROR,
+                                     GDK_PIXBUF_ERROR_CORRUPT_IMAGE,
+                                     _("BMP image has bogus header data"));
 		State->read_state = READ_STATE_ERROR;
 		return FALSE;
 	}
@@ -337,12 +386,10 @@ DecodeHeader (unsigned char *BFH, unsigned char *BIH,
 		if ((State->Header.width & 7) != 0)
 			State->LineWidth++;
 	} else {
-#if 0
-		g_set_error (error,
-			     GDK_PIXBUF_ERROR,
-			     GDK_PIXBUF_ERROR_CORRUPT_IMAGE,
-			     _("BMP image has bogus header data"));
-#endif
+		g_set_error_literal (error,
+                                     GDK_PIXBUF_ERROR,
+                                     GDK_PIXBUF_ERROR_CORRUPT_IMAGE,
+                                     _("BMP image has bogus header data"));
 		State->read_state = READ_STATE_ERROR;
 		return FALSE;
 	}
@@ -353,85 +400,139 @@ DecodeHeader (unsigned char *BFH, unsigned char *BIH,
 		State->LineWidth = (State->LineWidth / 4) * 4 + 4;
 
 	if (State->pixbuf == NULL) {
-		if (State->Type == 32 ||
-		    State->Compressed == BI_RLE4 ||
+		if (State->size_func) {
+			gint width = State->Header.width;
+			gint height = State->Header.height;
+
+			(*State->size_func) (&width, &height, State->user_data);
+			if (width == 0 || height == 0) {
+				State->read_state = READ_STATE_DONE;
+				State->BufferSize = 0;
+				return TRUE;
+			}
+		}
+
+		if (State->Type == 32 || 
+		    State->Compressed == BI_RLE4 || 
 		    State->Compressed == BI_RLE8)
 			State->pixbuf =
-			    gdk_pixbuf_new(GDK_COLORSPACE_RGB, TRUE, 8,
-					   (gint) State->Header.width,
-					   (gint) State->Header.height);
+				gdk_pixbuf_new(GDK_COLORSPACE_RGB, TRUE, 8,
+					       (gint) State->Header.width,
+					       (gint) State->Header.height);
 		else
 			State->pixbuf =
-			    gdk_pixbuf_new(GDK_COLORSPACE_RGB, FALSE, 8,
-					   (gint) State->Header.width,
-					   (gint) State->Header.height);
-
-                if (State->pixbuf == NULL) {
-#if 0
-                        g_set_error (error,
-                                     GDK_PIXBUF_ERROR,
-                                     GDK_PIXBUF_ERROR_INSUFFICIENT_MEMORY,
-                                     _("Not enough memory to load bitmap image"));
-#endif
+				gdk_pixbuf_new(GDK_COLORSPACE_RGB, FALSE, 8,
+					       (gint) State->Header.width,
+					       (gint) State->Header.height);
+		
+		if (State->pixbuf == NULL) {
+			g_set_error_literal (error,
+                                             GDK_PIXBUF_ERROR,
+                                             GDK_PIXBUF_ERROR_INSUFFICIENT_MEMORY,
+                                             _("Not enough memory to load bitmap image"));
 			State->read_state = READ_STATE_ERROR;
-                        return FALSE;
-                }
-
+			return FALSE;
+			}
+		
 		if (State->prepared_func != NULL)
 			/* Notify the client that we are ready to go */
-			(*State->prepared_func) (State->pixbuf, State->user_data);
-
+			(*State->prepared_func) (State->pixbuf, NULL, State->user_data);
+		
+		/* make all pixels initially transparent */
+		if (State->Compressed == BI_RLE4 || State->Compressed == BI_RLE8) {
+			memset (State->pixbuf->pixels, 0, State->pixbuf->rowstride * State->Header.height);
+			State->compr.p = State->pixbuf->pixels 
+				+ State->pixbuf->rowstride * (State->Header.height- 1);
+		}
 	}
-
-	/* make all pixels initially transparent */
-	if (State->Compressed == BI_RLE4 || State->Compressed == BI_RLE8) {
-		memset (State->pixbuf->pixels, 0, State->pixbuf->rowstride * State->Header.height);
-		State->compr.p = State->pixbuf->pixels 
-			+ State->pixbuf->rowstride * (State->Header.height- 1);
-	}
-
+	
 	State->BufferDone = 0;
 	if (State->Type <= 8) {
+                gint samples;
+
 		State->read_state = READ_STATE_PALETTE;
-		State->BufferSize = lsb_32 (&BFH[10]) - 14 - State->Header.size;
+
+		/* Allocate enough to hold the palette */
+	        samples = (State->Header.size == 12 ? 3 : 4);
+		State->BufferSize = State->Header.n_colors * samples;
+
+		/* Skip over everything between the palette and the data.
+		   This protects us against a malicious BFH[10] value.
+                */
+		State->BufferPadding = (lsb_32 (&BFH[10]) - 14 - State->Header.size) - State->BufferSize;
+
 	} else if (State->Compressed == BI_RGB) {
-		State->read_state = READ_STATE_DATA;
-		State->BufferSize = State->LineWidth;
+		if (State->BufferSize < lsb_32 (&BFH[10]))
+		{
+			/* skip over padding between headers and image data */
+			State->read_state = READ_STATE_HEADERS;
+			State->BufferDone = State->BufferSize;
+			State->BufferSize = lsb_32 (&BFH[10]);
+		}
+		else
+		{
+			State->read_state = READ_STATE_DATA;
+			State->BufferSize = State->LineWidth;
+		}
 	} else if (State->Compressed == BI_BITFIELDS) {
-		State->read_state = READ_STATE_BITMASKS;
-		State->BufferSize = 12;
+               if (State->Header.size == 108 || State->Header.size == 124) 
+               {
+			/* v4 and v5 have the bitmasks in the header */
+			if (!decode_bitmasks (&BIH[40], State, error)) {
+			       State->read_state = READ_STATE_ERROR;
+			       return FALSE;
+                        }
+               }
+               else 
+               {
+		       State->read_state = READ_STATE_BITMASKS;
+		       State->BufferSize = 12;
+               }
 	} else {
-#if 0
-		g_set_error (error,
-			     GDK_PIXBUF_ERROR,
-			     GDK_PIXBUF_ERROR_CORRUPT_IMAGE,
-			     _("BMP image has bogus header data"));
-#endif
+		g_set_error_literal (error,
+                                     GDK_PIXBUF_ERROR,
+                                     GDK_PIXBUF_ERROR_CORRUPT_IMAGE,
+                                     _("BMP image has bogus header data"));
 		State->read_state = READ_STATE_ERROR;
 		return FALSE;
 	}
 
-	if (!grow_buffer (State)) 
+	if (!grow_buffer (State, error)) 
 		return FALSE;
 
         return TRUE;
 }
 
-static gboolean
-DecodeColormap (guchar *buff,
-		struct bmp_progressive_state *State)
+static gboolean DecodeColormap (guchar *buff,
+				struct bmp_progressive_state *State,
+				GError **error)
 {
 	gint i;
+	gint samples;
 
 	g_assert (State->read_state == READ_STATE_PALETTE);
 
-	State->Colormap = g_malloc ((1 << State->Header.depth) * sizeof (*State->Colormap));
+	samples = (State->Header.size == 12 ? 3 : 4);
+	if (State->BufferSize < State->Header.n_colors * samples) {
+		State->BufferSize = State->Header.n_colors * samples;
+		if (!grow_buffer (State, error))
+			return FALSE;
+		return TRUE;
+	}
+
+	State->Colormap = g_malloc0 ((1 << State->Header.depth) * sizeof (*State->Colormap));
+	for (i = 0; i < State->Header.n_colors; i++)
 
-	for (i = 0; i < (1 << State->Header.depth); i++)
 	{
-		State->Colormap[i][0] = buff[i * (State->Header.size == 12 ? 3 : 4)];
-		State->Colormap[i][1] = buff[i * (State->Header.size == 12 ? 3 : 4) + 1];
-		State->Colormap[i][2] = buff[i * (State->Header.size == 12 ? 3 : 4) + 2];
+		State->Colormap[i][0] = buff[i * samples];
+		State->Colormap[i][1] = buff[i * samples + 1];
+		State->Colormap[i][2] = buff[i * samples + 2];
+#ifdef DUMPCMAP
+		g_print ("color %d %x %x %x\n", i,
+			 State->Colormap[i][0],
+			 State->Colormap[i][1],
+			 State->Colormap[i][2]);
+#endif
 	}
 
 	State->read_state = READ_STATE_DATA;
@@ -442,7 +543,7 @@ DecodeColormap (guchar *buff,
 	else
 		State->BufferSize = State->LineWidth;
 	
-	if (!grow_buffer (State))
+	if (!grow_buffer (State, error))
 		return FALSE;
 
 	return TRUE;
@@ -463,10 +564,13 @@ find_bits (int n, int *lowest, int *n_set)
 		}
 }
 
-/* Decodes the 3 shorts that follow for the bitmasks for BI_BITFIELDS coding */
+/* Decodes the bitmasks for BI_BITFIELDS coding */
 static gboolean
-decode_bitmasks (struct bmp_progressive_state *State, guchar *buf)
+decode_bitmasks (guchar *buf,
+		 struct bmp_progressive_state *State, 
+		 GError **error)
 {
+        State->a_mask = State->a_shift = State->a_bits = 0;
 	State->r_mask = buf[0] | (buf[1] << 8) | (buf[2] << 16) | (buf[3] << 24);
 	buf += 4;
 
@@ -479,21 +583,59 @@ decode_bitmasks (struct bmp_progressive_state *State, guchar *buf)
 	find_bits (State->g_mask, &State->g_shift, &State->g_bits);
 	find_bits (State->b_mask, &State->b_shift, &State->b_bits);
 
-	if (State->r_bits == 0 || State->g_bits == 0 || State->b_bits == 0) {
-		State->r_mask = 0x7c00;
-		State->r_shift = 10;
-		State->g_mask = 0x03e0;
-		State->g_shift = 5;
-		State->b_mask = 0x001f;
-		State->b_shift = 0;
+        /* v4 and v5 have an alpha mask */
+        if (State->Header.size == 108 || State->Header.size == 124) {
+	      buf += 4;
+	      State->a_mask = buf[0] | (buf[1] << 8) | (buf[2] << 16) | (buf[3] << 24);
+	      find_bits (State->a_mask, &State->a_shift, &State->a_bits);
+        }
 
-		State->r_bits = State->g_bits = State->b_bits = 5;
+	if (State->r_bits == 0 || State->g_bits == 0 || State->b_bits == 0) {
+                if (State->Type == 16) {
+		       State->r_mask = 0x7c00;
+		       State->r_shift = 10;
+		       State->g_mask = 0x03e0;
+		       State->g_shift = 5;
+		       State->b_mask = 0x001f;
+		       State->b_shift = 0;
+
+		       State->r_bits = State->g_bits = State->b_bits = 5;
+                }
+                else {
+		       State->r_mask = 0x00ff0000;
+		       State->r_shift = 16;
+		       State->g_mask = 0x0000ff00;
+		       State->g_shift = 8;
+		       State->b_mask = 0x000000ff;
+		       State->b_shift = 0;
+		       State->a_mask = 0xff000000;
+		       State->a_shift = 24;
+
+		       State->r_bits = State->g_bits = State->b_bits = State->a_bits = 8;
+                }
 	}
 
+        if (State->r_bits > 8) {
+          State->r_shift += State->r_bits - 8;
+          State->r_bits = 8;
+        }
+        if (State->g_bits > 8) {
+          State->g_shift += State->g_bits - 8;
+          State->g_bits = 8;
+        }
+        if (State->b_bits > 8) {
+          State->b_shift += State->b_bits - 8;
+          State->b_bits = 8;
+        }
+        if (State->a_bits > 8) {
+          State->a_shift += State->a_bits - 8;
+          State->a_bits = 8;
+        }
+
 	State->read_state = READ_STATE_DATA;
 	State->BufferDone = 0;
 	State->BufferSize = State->LineWidth;
-	if (!grow_buffer (State)) 
+	if (!grow_buffer (State, error)) 
 		return FALSE;
 
 	return TRUE;
@@ -505,16 +647,17 @@ decode_bitmasks (struct bmp_progressive_state *State, guchar *buf)
  * return context (opaque to user)
  */
 
-gpointer
-gdk_pixbuf__bmp_image_begin_load (ModulePreparedNotifyFunc prepared_func,
-				  ModuleUpdatedNotifyFunc updated_func,
-				  ModuleFrameDoneNotifyFunc frame_done_func,
-				  ModuleAnimationDoneNotifyFunc
-				  anim_done_func, gpointer user_data)
+static gpointer
+gdk_pixbuf__bmp_image_begin_load(GdkPixbufModuleSizeFunc size_func,
+                                 GdkPixbufModulePreparedFunc prepared_func,
+				 GdkPixbufModuleUpdatedFunc updated_func,
+                                 gpointer user_data,
+                                 GError **error)
 {
 	struct bmp_progressive_state *context;
-
+	
 	context = g_new0(struct bmp_progressive_state, 1);
+	context->size_func = size_func;
 	context->prepared_func = prepared_func;
 	context->updated_func = updated_func;
 	context->user_data = user_data;
@@ -522,12 +665,8 @@ gdk_pixbuf__bmp_image_begin_load (ModulePreparedNotifyFunc prepared_func,
 	context->read_state = READ_STATE_HEADERS;
 
 	context->BufferSize = 26;
-	context->buff = malloc (26);
-	if (!context->buff) {
-		g_free (context);
-		return NULL;
-	}
-
+	context->BufferPadding = 0;
+	context->buff = g_malloc(26);
 	context->BufferDone = 0;
 	/* 14 for the BitmapFileHeader, 12 for the BitmapImageHeader */
 
@@ -540,9 +679,9 @@ gdk_pixbuf__bmp_image_begin_load (ModulePreparedNotifyFunc prepared_func,
 	memset(&context->Header, 0, sizeof(struct headerpair));
 	memset(&context->compr, 0, sizeof(struct bmp_compression_state));
 
+
 	context->pixbuf = NULL;
-
-
+	
 	return (gpointer) context;
 }
 
@@ -551,32 +690,46 @@ gdk_pixbuf__bmp_image_begin_load (ModulePreparedNotifyFunc prepared_func,
  *
  * free context, unref gdk_pixbuf
  */
-void
-gdk_pixbuf__bmp_image_stop_load (gpointer data)
+static gboolean gdk_pixbuf__bmp_image_stop_load(gpointer data, GError **error)
 {
+	gboolean retval = TRUE;
+	
 	struct bmp_progressive_state *context =
 	    (struct bmp_progressive_state *) data;
 
+        /* FIXME this thing needs to report errors if
+         * we have unused image data
+         */
+	
+	g_return_val_if_fail(context != NULL, TRUE);
 
-	g_return_if_fail(context != NULL);
-
-	if (context->Colormap != NULL)
-		g_free(context->Colormap);
+	g_free(context->Colormap);
 
 	if (context->pixbuf)
-		gdk_pixbuf_unref(context->pixbuf);
+		g_object_unref(context->pixbuf);
 
-	free (context->buff);
-	g_free (context);
+	if (context->read_state == READ_STATE_HEADERS) {
+                if (error && *error == NULL) {
+                        g_set_error_literal (error,
+                                             GDK_PIXBUF_ERROR,
+                                             GDK_PIXBUF_ERROR_CORRUPT_IMAGE,
+                                             _("Premature end-of-file encountered"));
+                }
+		retval = FALSE;
+	}
+	
+	g_free(context->buff);
+	g_free(context);
+
+        return retval;
 }
 
 
 /*
 The OneLineXX functions are called when 1 line worth of data is present.
-OneLine32 is the 32 bpp-version.
+OneLine24 is the 24 bpp-version.
 */
-static void
-OneLine32 (struct bmp_progressive_state *context)
+static void OneLine32(struct bmp_progressive_state *context)
 {
 	int i;
 	guchar *pixels;
@@ -595,28 +748,35 @@ OneLine32 (struct bmp_progressive_state *context)
 		int r_lshift, r_rshift;
 		int g_lshift, g_rshift;
 		int b_lshift, b_rshift;
+		int a_lshift, a_rshift;
 
 		r_lshift = 8 - context->r_bits;
 		g_lshift = 8 - context->g_bits;
 		b_lshift = 8 - context->b_bits;
+		a_lshift = 8 - context->a_bits;
 
 		r_rshift = context->r_bits - r_lshift;
 		g_rshift = context->g_bits - g_lshift;
 		b_rshift = context->b_bits - b_lshift;
+		a_rshift = context->a_bits - a_lshift;
 
 		for (i = 0; i < context->Header.width; i++) {
-			int v, r, g, b;
+			unsigned int v, r, g, b, a;
 
-			v = src[0] | (src[1] << 8) | (src[2] << 16);
+			v = src[0] | (src[1] << 8) | (src[2] << 16) | (src[3] << 24);
 
 			r = (v & context->r_mask) >> context->r_shift;
 			g = (v & context->g_mask) >> context->g_shift;
 			b = (v & context->b_mask) >> context->b_shift;
+			a = (v & context->a_mask) >> context->a_shift;
 
 			*pixels++ = (r << r_lshift) | (r >> r_rshift);
 			*pixels++ = (g << g_lshift) | (g >> g_rshift);
 			*pixels++ = (b << b_lshift) | (b >> b_rshift);
-			*pixels++ = src[3]; /* alpha */
+                        if (context->a_bits)
+			  *pixels++ = (a << a_lshift) | (a >> a_rshift);
+                        else
+                          *pixels++ = 0xff;
 
 			src += 4;
 		}
@@ -625,14 +785,13 @@ OneLine32 (struct bmp_progressive_state *context)
 			*pixels++ = src[2];
 			*pixels++ = src[1];
 			*pixels++ = src[0];
-			*pixels++ = src[3];
+			*pixels++ = 0xff;
 
 			src += 4;
 		}
 }
 
-static void
-OneLine24 (struct bmp_progressive_state *context)
+static void OneLine24(struct bmp_progressive_state *context)
 {
 	gint X;
 	guchar *Pixels;
@@ -655,8 +814,7 @@ OneLine24 (struct bmp_progressive_state *context)
 
 }
 
-static void
-OneLine16 (struct bmp_progressive_state *context)
+static void OneLine16(struct bmp_progressive_state *context)
 {
 	int i;
 	guchar *pixels;
@@ -717,8 +875,7 @@ OneLine16 (struct bmp_progressive_state *context)
 		}
 }
 
-static void
-OneLine8 (struct bmp_progressive_state *context)
+static void OneLine8(struct bmp_progressive_state *context)
 {
 	gint X;
 	guchar *Pixels;
@@ -743,8 +900,7 @@ OneLine8 (struct bmp_progressive_state *context)
 	}
 }
 
-static void
-OneLine4 (struct bmp_progressive_state *context)
+static void OneLine4(struct bmp_progressive_state *context)
 {
 	gint X;
 	guchar *Pixels;
@@ -785,8 +941,7 @@ OneLine4 (struct bmp_progressive_state *context)
 
 }
 
-static void
-OneLine1 (struct bmp_progressive_state *context)
+static void OneLine1(struct bmp_progressive_state *context)
 {
 	gint X;
 	guchar *Pixels;
@@ -813,8 +968,7 @@ OneLine1 (struct bmp_progressive_state *context)
 }
 
 
-static void
-OneLine (struct bmp_progressive_state *context)
+static void OneLine(struct bmp_progressive_state *context)
 {
 	context->BufferDone = 0;
 	if (context->Lines >= context->Header.height)
@@ -862,16 +1016,26 @@ OneLine (struct bmp_progressive_state *context)
 #define END_OF_BITMAP 1
 #define DELTA         2
 
-static gboolean
-DoCompressed(struct bmp_progressive_state *context)
+static gboolean 
+DoCompressed(struct bmp_progressive_state *context, GError **error)
 {
 	gint i, j;
 	gint y;
 	guchar c;
 	gint idx;
 
-	if (context->compr.y >= context->Header.height)
+	/* context->compr.y might be past the last line because we are
+	 * on padding past the end of a valid data, or we might have hit
+	 * out-of-bounds data. Either way we just eat-and-ignore the
+	 * rest of the file. Doing the check only here and not when
+	 * we change y below is fine since BufferSize is always 2 here
+	 * and the BMP file format always starts new data on 16-bit
+	 * boundaries.
+	 */
+	if (context->compr.y >= context->Header.height) {
+		context->BufferDone = 0;
 		return TRUE;
+	}
 
 	y = context->compr.y;
 
@@ -992,12 +1156,15 @@ DoCompressed(struct bmp_progressive_state *context)
 	}
 	if (context->updated_func != NULL) {
 		if (context->compr.y > y)
+		{
+			gint new_y = MIN (context->compr.y, context->Header.height);
 			(*context->updated_func) (context->pixbuf,
 						  0,
-						  y,
+						  context->Header.height - new_y,
 						  context->Header.width,
-						  context->compr.y - y,
+						  new_y - y,
 						  context->user_data);
+		}
 
 	}
 
@@ -1010,14 +1177,19 @@ DoCompressed(struct bmp_progressive_state *context)
  * buf - new image data
  * size - length of new image data
  *
- * append image data onto inrecrementally built output image
+ * append image data onto incrementally built output image
  */
-gboolean
-gdk_pixbuf__bmp_image_load_increment (gpointer data, guchar *buf, guint size)
+static gboolean
+gdk_pixbuf__bmp_image_load_increment(gpointer data,
+                                     const guchar * buf,
+                                     guint size,
+                                     GError **error)
 {
-	struct bmp_progressive_state *context = (struct bmp_progressive_state *) data;
+	struct bmp_progressive_state *context =
+	    (struct bmp_progressive_state *) data;
 
 	gint BytesToCopy;
+	gint BytesToRemove;
 
 	if (context->read_state == READ_STATE_DONE)
 		return TRUE;
@@ -1043,30 +1215,48 @@ gdk_pixbuf__bmp_image_load_increment (gpointer data, guchar *buf, guint size)
 				break;
 		}
 
+		/* context->buff is full.  Now we discard all "padding" */
+		if (context->BufferPadding != 0) {
+			BytesToRemove = context->BufferPadding - size;
+			if (BytesToRemove > size) {
+				BytesToRemove = size;
+			}
+			size -= BytesToRemove;
+			context->BufferPadding -= BytesToRemove;
+
+			if (context->BufferPadding != 0)
+				break;
+		}
+
 		switch (context->read_state) {
 		case READ_STATE_HEADERS:
-			if (!DecodeHeader (context->buff, context->buff + 14, context))
+			if (!DecodeHeader (context->buff,
+					   context->buff + 14, context,
+					   error))
 				return FALSE;
 
 			break;
 
 		case READ_STATE_PALETTE:
-			if (!DecodeColormap (context->buff, context))
+			if (!DecodeColormap (context->buff, context, error))
 				return FALSE;
 			break;
 
 		case READ_STATE_BITMASKS:
-			if (!decode_bitmasks (context, context->buff))
+			if (!decode_bitmasks (context->buff, context, error))
 				return FALSE;
 			break;
 
 		case READ_STATE_DATA:
 			if (context->Compressed == BI_RGB || context->Compressed == BI_BITFIELDS)
 				OneLine (context);
-			else if (!DoCompressed (context))
+			else if (!DoCompressed (context, error))
 				return FALSE;
 
 			break;
+		case READ_STATE_DONE:
+			return TRUE;
+			break;
 
 		default:
 			g_assert_not_reached ();
@@ -1075,3 +1265,164 @@ gdk_pixbuf__bmp_image_load_increment (gpointer data, guchar *buf, guint size)
 
 	return TRUE;
 }
+
+/* for our convenience when filling file header */
+#define put16(buf,data)	{ guint16 x; \
+			  x = GUINT16_TO_LE (data); \
+			  memcpy(buf, &x, 2); \
+			  buf += 2; }
+#define put32(buf,data)	{ guint32 x; \
+			  x = GUINT32_TO_LE (data); \
+			  memcpy(buf, &x, 4); \
+			  buf += 4; }
+
+static gboolean
+gdk_pixbuf__bmp_image_save_to_callback (GdkPixbufSaveFunc   save_func,
+					gpointer            user_data,
+					GdkPixbuf          *pixbuf, 
+					gchar             **keys,
+					gchar             **values,
+					GError            **error)
+{
+	guint width, height, channel, size, stride, src_stride, x, y;
+	guchar BFH_BIH[54], *pixels, *buf, *src, *dst, *dst_line;
+	gboolean ret;
+
+	width = gdk_pixbuf_get_width (pixbuf);
+	height = gdk_pixbuf_get_height (pixbuf);
+	channel = gdk_pixbuf_get_n_channels (pixbuf);
+	pixels = gdk_pixbuf_get_pixels (pixbuf);
+	src_stride = gdk_pixbuf_get_rowstride (pixbuf);
+	stride = (width * 3 + 3) & ~3;
+	size = stride * height;
+
+	/* filling BFH */
+	dst = BFH_BIH;
+	*dst++ = 'B';			/* bfType */
+	*dst++ = 'M';
+	put32 (dst, size + 14 + 40);	/* bfSize */
+	put32 (dst, 0);			/* bfReserved1 + bfReserved2 */
+	put32 (dst, 14 + 40);		/* bfOffBits */
+
+	/* filling BIH */
+	put32 (dst, 40);		/* biSize */
+	put32 (dst, width);		/* biWidth */
+	put32 (dst, height);		/* biHeight */
+	put16 (dst, 1);			/* biPlanes */
+	put16 (dst, 24);		/* biBitCount */
+	put32 (dst, BI_RGB);		/* biCompression */
+	put32 (dst, size);		/* biSizeImage */
+	put32 (dst, 0);			/* biXPelsPerMeter */
+	put32 (dst, 0);			/* biYPelsPerMeter */
+	put32 (dst, 0);			/* biClrUsed */
+	put32 (dst, 0);			/* biClrImportant */
+
+	if (!save_func ((gchar *)BFH_BIH, 14 + 40, error, user_data))
+		return FALSE;
+
+	dst_line = buf = g_try_malloc (size);
+	if (!buf) {
+		g_set_error_literal (error,
+                                     GDK_PIXBUF_ERROR,
+                                     GDK_PIXBUF_ERROR_INSUFFICIENT_MEMORY,
+                                     _("Couldn't allocate memory for saving BMP file"));
+		return FALSE;
+	}
+
+	/* saving as a bottom-up bmp */
+	pixels += (height - 1) * src_stride;
+	for (y = 0; y < height; ++y, pixels -= src_stride, dst_line += stride) {
+		dst = dst_line;
+		src = pixels;
+		for (x = 0; x < width; ++x, dst += 3, src += channel) {
+			dst[0] = src[2];
+			dst[1] = src[1];
+			dst[2] = src[0];
+		}
+	}
+	ret = save_func ((gchar *)buf, size, error, user_data);
+	g_free (buf);
+
+	return ret;
+}
+
+static gboolean
+save_to_file_cb (const gchar *buf,
+		 gsize count,
+		 GError **error,
+		 gpointer data)
+{
+	gint bytes;
+	
+	while (count > 0) {
+		bytes = fwrite (buf, sizeof (gchar), count, (FILE *) data);
+		if (bytes <= 0)
+			break;
+		count -= bytes;
+		buf += bytes;
+	}
+
+	if (count) {
+		g_set_error_literal (error,
+                                     GDK_PIXBUF_ERROR,
+                                     GDK_PIXBUF_ERROR_FAILED,
+                                     _("Couldn't write to BMP file"));
+		return FALSE;
+	}
+	
+	return TRUE;
+}
+
+static gboolean
+gdk_pixbuf__bmp_image_save (FILE          *f, 
+                            GdkPixbuf     *pixbuf, 
+                            gchar        **keys,
+                            gchar        **values,
+                            GError       **error)
+{
+	return gdk_pixbuf__bmp_image_save_to_callback (save_to_file_cb,
+						       f, pixbuf, keys,
+						       values, error);
+}
+
+#ifndef INCLUDE_bmp
+#define MODULE_ENTRY(function) G_MODULE_EXPORT void function
+#else
+#define MODULE_ENTRY(function) void _gdk_pixbuf__bmp_ ## function
+#endif
+
+MODULE_ENTRY (fill_vtable) (GdkPixbufModule *module)
+{
+	module->begin_load = gdk_pixbuf__bmp_image_begin_load;
+	module->stop_load = gdk_pixbuf__bmp_image_stop_load;
+	module->load_increment = gdk_pixbuf__bmp_image_load_increment;
+	module->save = gdk_pixbuf__bmp_image_save;
+	module->save_to_callback = gdk_pixbuf__bmp_image_save_to_callback;
+}
+
+MODULE_ENTRY (fill_info) (GdkPixbufFormat *info)
+{
+	static GdkPixbufModulePattern signature[] = {
+		{ "BM", NULL, 100 },
+		{ NULL, NULL, 0 }
+	};
+	static gchar * mime_types[] = {
+		"image/bmp",
+		"image/x-bmp",
+		"image/x-MS-bmp",
+		NULL
+	};
+	static gchar * extensions[] = {
+		"bmp",
+		NULL
+	};
+
+	info->name = "bmp";
+	info->signature = signature;
+	info->description = N_("The BMP image format");
+	info->mime_types = mime_types;
+	info->extensions = extensions;
+	info->flags = GDK_PIXBUF_FORMAT_WRITABLE | GDK_PIXBUF_FORMAT_THREADSAFE;
+	info->license = "LGPL";
+}
+
